/* File: ArrayList.h.
   Author:  T.S. Norvell
   Update:  0.  5/May/2004.  First revision.
   Update:  1.  7/May/2004.  Changed length to 500.

/* ADT List -- Implemented using an array.
   Header file.
*/
#ifndef ARRAY_LIST_H
#define ARRAY_LIST_H


template <class T>
class List  {
	public :

	// Exceptions

		class Space_Exception{} ;

	// Creators

		List() ;
			// (Constructor) Create a list. Initially it is empty.

		// List(const list &l) ;
			// (Copy constructor.) Create a copy of list l.

	// Destroyers

		//~List() ;
			// Destroy this list.

	// Accessors

		int length() const;
			// Returns the length of this list.

		T retrieve(int i) const;
			// Returns item number i from this list.
			// Items are numbered from 0, so i should obey
			// 0 _< i < the length of this list.

        T &retrieveRef(int i) ;
            // Returns a reference to item number i from this list.
            // Precondition: 0 _< i < the length of this list.
            // Warning: the returned reference will become invalid if
            // any mutator is called.

	// Mutators
		// const List& operator=(const List &l) ;
			// (Assignment operator.) Copy l over this list.

		void insert(int i, T newData ) ;
			// Insert newData into this list at position i.

			/* The data originally at position i (if any) and all
			 data at higher positions should be shifted up one
			 position, so the data originally at position i will
			 be found at position i+1, etc. Note i should obey
			 0 _< i _< the length of this list. If the data can
			 not be inserted, e.g. because there is no space, then
			 Space_Exception should be thrown. */

		void del(int i) ;
			// Delete the data at position i.
			/* The data at higher positions (if any) should be
			  shifted down, so for example the data at position
			  i+1, should end up at position i.  Note i should obey
			  0 _< i< the length of this list. */

	private:

		/* The list is represented by the contents of array `a'. The
		   contents of list position i are stored in location a[i].
		   The current length on the list is stored in `len'. When
		   `len==maxLen' the list is full and nothing else can be
		   stored. */

		// static const int maxLen = 500 ; // This is the ISO syntax.
        enum { maxLen = 500 } ; // Hack to declare a constant.

		int len_ ;
		T a_[ maxLen ] ;
} ;


////////////////////////////////////////////
// IMPLEMENTATION                         //
////////////////////////////////////////////

#include <cassert>

// Default Constructor
template <class T>
List<T> :: List() {
	// Initialize the members.
	len_ = 0 ; }

// length()
template <class T>
int List<T> :: length() const {
    return len_ ; }

// retrieve(i)
template <class T>
T List<T>:: retrieve (int i) const {
	// Check precondition
	assert( 0 <= i && i < len_ ) ;
	return a_[i] ; }

// retrieveRef(i)
template <class T>
T& List<T>:: retrieveRef (int i) {
	// Check precondition
	assert( 0 <= i && i < len_ ) ;
	return a_[i] ; }

// insert(i, newData)
template <class T>
void List<T>:: insert (int i, T newData) {
	// Check precodition
		assert( 0 <= i && i <= len_ ) ;
	
    if( len_ == maxLen ) {
		throw Space_Exception() ; }
	else {
		// Make space by pushing all entries from i on up
		// one spot to the right.
		    for( int k = len_; k > i ; --k ) {
			    a_[k] = a_[k-1] ; }
        
        // Copy in the new data
		    a_[i] = newData ;
        // Adjust the length
            len_ += 1 ; } }

// del(i)
template <class T>
void List<T>:: del( int i ) {
    // Check Precodition
        assert( 0 <= i && i < len_ ) ;
    
    // Adjust the length
        len_ -= 1 ;

    // Push all entries from i on up, one space to the left.
        for( int k = i ; k < len_ ; ++k ) {
            a_[k] = a_[k+1] ; } }

#endif
